import time
import math
import sys

import ac
import acsys

import helisession
import heliSettings
import helicar
import euclid

sys.path.insert(0,  'apps/python/helicorsa/dll')

appWindow=0
lastTimeUpdate=0
running = True

hSession = None

showWindowTitle = True
appWindowActivated = 0

currentBackgroundAlpha = 0

sharedMemoryLoaded = False
isRaceSession = True

#debug output label, ignore this 
dbgLabel = 0

lblIndicatorLeft=0
lblIndicatorRight=0
lastIndicatorLeft=0
lastIndicatorRight=0


# load sim_info: code by ferito
try:
    from third_party import sim_info
    sharedMemoryLoaded  = True
except Exception as e:
    ac.log("helicorsa::error while import sim_info. %s" % e)
    ac.log("helicorsa:: can't tell wether race or p/q, so blueflag is always enabled")

def acMain(ac_version):
    global appWindow, hSession, dbgLabel, lblIndicatorLeft, lblIndicatorRight

    try:
        z = heliSettings.guiZoomFactor

        appWindow=ac.newApp("helicorsa")
        ac.setSize(appWindow,200*z,200*z)
        ac.drawBorder(appWindow,0)
        ac.setBackgroundOpacity(appWindow,0.5)
        ac.setTitle(appWindow, heliSettings.helicorsaTitle)

        ac.addRenderCallback(appWindow , onFormRender)
        ac.addOnAppActivatedListener(appWindow,onAppActivated)
        appWindowActivated = time.clock()
        showWindowTitle = True

        dbgLabel = ac.addLabel(appWindow, "")
        ac.setPosition(dbgLabel, 15, 15)

        hSession = helisession.HeliSession()

        lblIndicatorLeft = ac.addLabel(appWindow, "")
        ac.setSize(lblIndicatorLeft, 100*z,152*z)
        ac.setPosition(lblIndicatorLeft, 0, heliSettings.yOff*z - 152*z / 2)

        lblIndicatorRight = ac.addLabel(appWindow, "")
        ac.setSize(lblIndicatorRight, 100*z,152*z)
        ac.setPosition(lblIndicatorRight, heliSettings.xOff*z, heliSettings.yOff*z - 152*z / 2)

        ac.log("helicorsa::acMain finished")
    except Exception as e:
        ac.log("helicorsa::acMain() %s" % e)
    return "helicorsa"

def onAppActivated(*args):
    global appWindowActivated, showWindowTitle
    ac.log("helicorsa::onAppActivated({0})".format(args))
    appWindowActivated = time.clock()
    showWindowTitle = True
    ac.setBackgroundOpacity(appWindow, 0.5)
    ac.setIconPosition(appWindow, 0, 0)
    ac.setTitle(appWindow, heliSettings.helicorsaTitle)

    
def acShutdown():
    running = False
    hSession = None
    centerOffset = None
    ac.removeItem(appWindow)

def timeForCalculationCame():
    global lastTimeUpdate
    now = time.clock()

    # check if updateThreshold reached
    if (lastTimeUpdate == 0) or (now - lastTimeUpdate > heliSettings.updateThreshold):
        lastTimeUpdate = now
        return True
    else:
        return False

def doCalculationStuff():
    global hSession, sharedMemoryLoaded, isRaceSession

    #Phase 0: Is this a race session?
    if sharedMemoryLoaded:
        # get shared memory info
        sim_info_obj = sim_info.SimInfo()
        #ac.log("session_type is %d" % (sim_info_obj.graphics.session))
        isRaceSession = sim_info_obj.graphics.session == sim_info.AC_RACE

    #Phase 1: Detect all the players online 
    #--------------------------------------------------------------------------------------------
    #1. First of all we have to detect new players and players left (since the last tick)
    #--------------------------------------------------------------------------------------------
    # stolen by ferito-LiveCarTracker:
    players = {}
    carIds = range(1, 24, 1)
    for carId in carIds:
        driverName = ac.getDriverName(carId)
        # if real car, register it
        if type(driverName) == type(""):
            players[carId] = driverName
    #now we have a dictionary { id : name } of all current cars. Let the session know this.
    anythingChanged = hSession.updateCurrentPlayers(players)

    #then all the cars will receive their geo-update
    hSession.calcWorldPositions(0)

    #debug label: quick live info for development
    #ac.setText(dbgLabel, "player 1 iL: {0}, iR :{1}".format(hSession.cars[1].overlapIndicatorL, hSession.cars[1].overlapIndicatorR))


    #Hotfix: Hide the background when clicked. I really hope this won't affect performance
    if heliSettings.removeBackgroundEveryFrame == 1 and showWindowTitle == False:
        ac.setBackgroundOpacity(appWindow, 0)


def onFormRender(deltaT):
    global showWindowTitle, appWindowActivated, appWindow, dbgLabel
    if running == False:
        return

    if showWindowTitle:
        if (time.clock() - appWindowActivated > heliSettings.showTitle):
            showWindowTitle = False
            ac.setBackgroundOpacity(appWindow, 0)
            ac.setIconPosition(appWindow, -7000, -3000)
            ac.setTitle(appWindow, "")

    try:
        #we won't do all the calculations every time, so we have to sort out some frames.
        #But: We'll need the graphics stuff every single time, or we'll have flickering. But no worry, opengl is fast

        if timeForCalculationCame():
            doCalculationStuff()

        #Now we draw the current cars on the minimap        
        drawCars()
    
    except Exception as e:
        ac.log("helicorsa::onFormRender() %s" % e)

def drawIndicators(maxIndicatorL, maxIndicatorR):
    global lblIndicatorLeft, lblIndicatorRight, lastIndicatorLeft, lastIndicatorRight

    if heliSettings.showIndicators:
        if maxIndicatorL != lastIndicatorLeft:
            lastIndicatorLeft = maxIndicatorL
            if maxIndicatorL == 0:
                ac.setBackgroundTexture(lblIndicatorLeft, "apps/python/helicorsa/img/empty.png")
            if maxIndicatorL == 1:
                ac.setBackgroundTexture(lblIndicatorLeft, "apps/python/helicorsa/img/heco_yellowL.png")
            if maxIndicatorL == 2:
                ac.setBackgroundTexture(lblIndicatorLeft, "apps/python/helicorsa/img/heco_orangeL.png")
        if maxIndicatorR != lastIndicatorRight:
            lastIndicatorRight = maxIndicatorR
            if maxIndicatorR == 0:
                ac.setBackgroundTexture(lblIndicatorRight, "apps/python/helicorsa/img/empty.png")
            if maxIndicatorR == 1:
                ac.setBackgroundTexture(lblIndicatorRight, "apps/python/helicorsa/img/heco_yellowR.png")
            if maxIndicatorR == 2:
                ac.setBackgroundTexture(lblIndicatorRight, "apps/python/helicorsa/img/heco_orangeR.png")



def drawCars():
    global hSession, currentBackgroundAlpha, isRaceSession

    #basicall we iterate all the (near) cars and try to draw points or 
    #whatever shapes of them - relative to the player's position and direction

    blueflagPlayer = False
    maxOpacity = 0.0

    #first we check wether to show any indicators
    if heliSettings.showIndicators:
        maxIndicatorL = 0
        maxIndicatorR = 0
        for c in hSession.nearcars:
            if c.overlapIndicatorL > maxIndicatorL:
                maxIndicatorL = c.overlapIndicatorL
            if c.overlapIndicatorR > maxIndicatorR:
                maxIndicatorR = c.overlapIndicatorR

        drawIndicators(maxIndicatorL, maxIndicatorR)


    if heliSettings.showCars:
        #then we draw all near cars
        for c in hSession.nearcars:
            #aaand draw. Colors depends on the lap count compared to the player.
            r, g, b = 1, 1, 1

            if isRaceSession and c.laps > hSession.player.laps and c.splineposition < hSession.player.splineposition and hSession.player.splineposition - c.splineposition < 0.05:
                r = 1
                g = 0
                b = 0
                blueflagPlayer = True

            if heliSettings.carBorderThickness > 0:
                c.drawYourselfRoundedWithBorders(r,g,b, 0.5, 0.5, 0.5)
            else:
                c.drawYourself(r, g, b)

            if c.maxOpacity > maxOpacity:
                maxOpacity = c.maxOpacity
    
        #then we'll draw the players car at the center.
        hSession.player.maxOpacity = maxOpacity
        if isRaceSession and blueflagPlayer:
            hSession.player.drawYourselfRoundedWithBorders(0, 0, 1, 0.5, 0.5, 0.5)
        else:
            hSession.player.drawYourselfRoundedWithBorders(1, 1, 1, 0.5, 0.8, 0.8)

    #then we have a background
    if heliSettings.showBackgroundPic == 1:
        alpha = round(maxOpacity*10)
        if alpha != currentBackgroundAlpha:
            currentBackgroundAlpha = alpha
            ac.setBackgroundTexture(appWindow, "apps/python/helicorsa/img/bg" + str(alpha) + ".png")

